@@ -943,12 +943,10 @@ draw_line_markers (GtkSourceView *view,
 		window = gtk_text_view_get_window (GTK_TEXT_VIEW (view),
 						   GTK_TEXT_WINDOW_LEFT);
 
-		gdk_pixbuf_render_to_drawable_alpha (composite,
-						     GDK_DRAWABLE (window),
-						     0, 0, x, y,
-						     width, height,
-						     GDK_PIXBUF_ALPHA_BILEVEL,
-						     127, GDK_RGB_DITHER_NORMAL, 0, 0);
+		gdk_draw_pixbuf (GDK_DRAWABLE (window), NULL, composite,
+				 0, 0, x, y,
+				 width, height,
+				 GDK_RGB_DITHER_NORMAL, 0, 0);
 		g_object_unref (composite);
 	}
 
@@ -1187,7 +1185,7 @@ gtk_source_view_expose (GtkWidget      *widget,
 		gtk_text_iter_forward_line (&iter2);
 
 		_gtk_source_buffer_highlight_region (view->priv->source_buffer,
-						     &iter1, &iter2);
+						     &iter1, &iter2, FALSE);
 	}
 
 	/* now check for the left window, which contains the margin */
@@ -1821,6 +1819,7 @@ gtk_source_view_set_margin (GtkSourceView *view, guint margin)
 		return;
 
 	view->priv->margin = margin;
+	view->priv->cached_margin_width = -1;
 
 	gtk_widget_queue_draw (GTK_WIDGET (view));
 
